// ============================================================================
// Project:	Deductive Database
// Filename:	cur_ii.cpp
// Purpose:	Cursor for accessing rel_ii (full table scan).
// Last Change:	11.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "str.h"
#ifdef	VER_DUMP
#include "err.h"
#endif
#include "check.h"
#include "rel_ii.h"

#include "cur_ii.h"


//=============================================================================
// Constructor and Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

cur_ii_c::cur_ii_c(rel_ii_t rel)
{
	// Set Magic number:
	CHECK_CODE(Magic = CUR_II_MAGIC);

	// Initialize attributes:
	Rel = rel;
	Pos = -2;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

cur_ii_c::~cur_ii_c()
{
	// Check this object:
	CHECK_VALID("cur_ii_c::destructor");

	// Make this object invalid:
	CHECK_CODE(Magic = 0);
}


//=============================================================================
// Debugging Support:
//=============================================================================

//-----------------------------------------------------------------------------
// check: Check the integrity of this object, return error message or NULL.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
str_t	cur_ii_c::check() const
{
	// Check magic number:
	if(Magic != CUR_II_MAGIC)
		return "wrong magic number";

	// Check pointer to relation:
	if(Rel == REL_II_NULL)
		return "cursor with null relation";

	// Check position:
	if(Pos < -2)
		return "invalid position (too small)";
	if(Pos == -1)
		return "invalid position (-1)";
	if(Pos > Rel->TupleArr.length())
		return "invalid position (too large)";
	if(Pos != -2 && Pos % 2 != 0)
		return "invalid position (not even)";

	// Ok:
	return 0;
}
#endif


//-----------------------------------------------------------------------------
// dump: Show data structure.
//-----------------------------------------------------------------------------

#ifdef	VER_DUMP
void	cur_ii_c::dump(str_t headline) const
{
	// Headline:
	if(headline == STR_NULL)
		headline = "Cursor for rel_ii (Relation with two integer cols)";
	err_c::dump_begin(headline);

	// Relation name:
	err_c::dump_str("Relation:        '");
	err_c::dump_str(Rel->name());
	err_c::dump_str("'");
	err_c::dump_nl();

	// Current position:
	err_c::dump_str("Current Position: ");
	err_c::dump_int(Pos);
	if(Pos == -2)
		err_c::dump_str("  [Before first tuple]");
	else if(Pos > Rel->TupleArr.length())
		err_c::dump_str("  [At End (after last tuple)]");
	else {
		err_c::dump_str("  [At Tuple: ");
		err_c::dump_int(Pos/2);
		err_c::dump_str("]");
	}
	err_c::dump_nl();

	// Size of this object:
	err_c::dump_str("Object Size:   ");
	err_c::dump_uint(static_cast<unsigned int>(sizeof(cur_ii_c)));
	err_c::dump_str(" Bytes");
	err_c::dump_nl();
	err_c::dump_nl();

	// Done:
	err_c::dump_end();
}
#endif

